"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from gr4vy.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Dict
from typing_extensions import NotRequired, TypedDict


class CybersourceAntiFraudOptionsTypedDict(TypedDict):
    merchant_defined_data: NotRequired[Nullable[Dict[str, str]]]
    r"""A list of merchant defined data to be passed to the Cybersource Decision Manager API. Each key needs to be a numeric string."""
    meta_key_merchant_id: NotRequired[Nullable[str]]
    r"""The merchant ID to use for this transaction. This requires a meta key to be set up for use with Cybersource Decision Manager, and this overrides the connector configuration."""
    shipping_method: NotRequired[Nullable[str]]
    r"""The shipping method for this transaction."""


class CybersourceAntiFraudOptions(BaseModel):
    merchant_defined_data: OptionalNullable[Dict[str, str]] = UNSET
    r"""A list of merchant defined data to be passed to the Cybersource Decision Manager API. Each key needs to be a numeric string."""

    meta_key_merchant_id: OptionalNullable[str] = UNSET
    r"""The merchant ID to use for this transaction. This requires a meta key to be set up for use with Cybersource Decision Manager, and this overrides the connector configuration."""

    shipping_method: OptionalNullable[str] = UNSET
    r"""The shipping method for this transaction."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            ["merchant_defined_data", "meta_key_merchant_id", "shipping_method"]
        )
        nullable_fields = set(
            ["merchant_defined_data", "meta_key_merchant_id", "shipping_method"]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            is_nullable_and_explicitly_set = (
                k in nullable_fields
                and (self.__pydantic_fields_set__.intersection({n}))  # pylint: disable=no-member
            )

            if val != UNSET_SENTINEL:
                if (
                    val is not None
                    or k not in optional_fields
                    or is_nullable_and_explicitly_set
                ):
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .threedsecurescenarioconditions import (
    ThreeDSecureScenarioConditions,
    ThreeDSecureScenarioConditionsTypedDict,
)
from .threedsecurescenariooutcome import (
    ThreeDSecureScenarioOutcome,
    ThreeDSecureScenarioOutcomeTypedDict,
)
from gr4vy.types import BaseModel
from typing_extensions import TypedDict


class ThreeDSecureScenarioCreateTypedDict(TypedDict):
    conditions: ThreeDSecureScenarioConditionsTypedDict
    outcome: ThreeDSecureScenarioOutcomeTypedDict


class ThreeDSecureScenarioCreate(BaseModel):
    conditions: ThreeDSecureScenarioConditions

    outcome: ThreeDSecureScenarioOutcome



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from gr4vy.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
    UnrecognizedStr,
)
from pydantic import model_serializer
from typing import Literal, Union
from typing_extensions import NotRequired, TypedDict


PurposeCode = Union[
    Literal[
        "mortgage",
        "utility",
        "loan",
        "dependant_support",
        "gambling",
        "retail",
        "salary",
        "personal",
        "government",
        "pension",
        "tax",
        "other",
    ],
    UnrecognizedStr,
]


class GoCardlessOptionsTypedDict(TypedDict):
    purpose_code: NotRequired[Nullable[PurposeCode]]
    r"""Specifies the high-level purpose of a mandate and/or payment using a set of pre-defined categories. Required for the PayTo scheme, optional for all others"""


class GoCardlessOptions(BaseModel):
    purpose_code: OptionalNullable[PurposeCode] = UNSET
    r"""Specifies the high-level purpose of a mandate and/or payment using a set of pre-defined categories. Required for the PayTo scheme, optional for all others"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["purpose_code"])
        nullable_fields = set(["purpose_code"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            is_nullable_and_explicitly_set = (
                k in nullable_fields
                and (self.__pydantic_fields_set__.intersection({n}))  # pylint: disable=no-member
            )

            if val != UNSET_SENTINEL:
                if (
                    val is not None
                    or k not in optional_fields
                    or is_nullable_and_explicitly_set
                ):
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .address import Address, AddressTypedDict
from gr4vy.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from pydantic import model_serializer
from typing_extensions import NotRequired, TypedDict


class ShippingDetailsUpdateTypedDict(TypedDict):
    first_name: NotRequired[Nullable[str]]
    r"""The first name(s) or given name for the buyer."""
    last_name: NotRequired[Nullable[str]]
    r"""The last name, or family name, of the buyer."""
    email_address: NotRequired[Nullable[str]]
    r"""The email address for the buyer."""
    phone_number: NotRequired[Nullable[str]]
    r"""The phone number for the buyer which should be formatted according to the E164 number standard."""
    address: NotRequired[Nullable[AddressTypedDict]]
    r"""The billing address for the buyer."""


class ShippingDetailsUpdate(BaseModel):
    first_name: OptionalNullable[str] = UNSET
    r"""The first name(s) or given name for the buyer."""

    last_name: OptionalNullable[str] = UNSET
    r"""The last name, or family name, of the buyer."""

    email_address: OptionalNullable[str] = UNSET
    r"""The email address for the buyer."""

    phone_number: OptionalNullable[str] = UNSET
    r"""The phone number for the buyer which should be formatted according to the E164 number standard."""

    address: OptionalNullable[Address] = UNSET
    r"""The billing address for the buyer."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            ["first_name", "last_name", "email_address", "phone_number", "address"]
        )
        nullable_fields = set(
            ["first_name", "last_name", "email_address", "phone_number", "address"]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            is_nullable_and_explicitly_set = (
                k in nullable_fields
                and (self.__pydantic_fields_set__.intersection({n}))  # pylint: disable=no-member
            )

            if val != UNSET_SENTINEL:
                if (
                    val is not None
                    or k not in optional_fields
                    or is_nullable_and_explicitly_set
                ):
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .transaction import Transaction, TransactionTypedDict
from .voidstatus import VoidStatus
from gr4vy.types import BaseModel, Nullable, UNSET_SENTINEL
from gr4vy.utils import validate_const
import pydantic
from pydantic import model_serializer
from pydantic.functional_validators import AfterValidator
from typing import Literal, Optional
from typing_extensions import Annotated, TypedDict


class TransactionVoidTypedDict(TypedDict):
    status: VoidStatus
    code: Nullable[str]
    r"""The standardized error code set by Gr4vy."""
    raw_response_code: Nullable[str]
    r"""This is the response code received from the payment service. This can be set to any value and is not standardized across different payment services."""
    raw_response_description: Nullable[str]
    r"""This is the response description received from the payment service. This can be set to any value and is not standardized across different payment services."""
    transaction: TransactionTypedDict
    r"""A full transaction resource."""
    type: Literal["transaction-void"]
    r"""Always `transaction-void`."""


class TransactionVoid(BaseModel):
    status: VoidStatus

    code: Nullable[str]
    r"""The standardized error code set by Gr4vy."""

    raw_response_code: Nullable[str]
    r"""This is the response code received from the payment service. This can be set to any value and is not standardized across different payment services."""

    raw_response_description: Nullable[str]
    r"""This is the response description received from the payment service. This can be set to any value and is not standardized across different payment services."""

    transaction: Transaction
    r"""A full transaction resource."""

    TYPE: Annotated[
        Annotated[
            Optional[Literal["transaction-void"]],
            AfterValidator(validate_const("transaction-void")),
        ],
        pydantic.Field(alias="type"),
    ] = "transaction-void"
    r"""Always `transaction-void`."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["type"])
        nullable_fields = set(["code", "raw_response_code", "raw_response_description"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            is_nullable_and_explicitly_set = (
                k in nullable_fields
                and (self.__pydantic_fields_set__.intersection({n}))  # pylint: disable=no-member
            )

            if val != UNSET_SENTINEL:
                if (
                    val is not None
                    or k not in optional_fields
                    or is_nullable_and_explicitly_set
                ):
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from gr4vy.types import BaseModel, UNSET_SENTINEL
from gr4vy.utils import validate_const
import pydantic
from pydantic import model_serializer
from pydantic.functional_validators import AfterValidator
from typing import Literal, Optional
from typing_extensions import Annotated, TypedDict


class CryptogramTypedDict(TypedDict):
    cryptogram: str
    r"""The cryptogram of the network token."""
    type: Literal["network-token-cryptogram"]
    r"""Always `network-token-cryptogram`."""


class Cryptogram(BaseModel):
    cryptogram: str
    r"""The cryptogram of the network token."""

    TYPE: Annotated[
        Annotated[
            Optional[Literal["network-token-cryptogram"]],
            AfterValidator(validate_const("network-token-cryptogram")),
        ],
        pydantic.Field(alias="type"),
    ] = "network-token-cryptogram"
    r"""Always `network-token-cryptogram`."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["type"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from gr4vy.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing_extensions import Annotated, NotRequired, TypedDict


class NuveiPSEOptionsTypedDict(TypedDict):
    user_type: NotRequired[Nullable[str]]
    r"""Customer type (\"N\" for persona natural, \"J\" for persona jurídica)"""
    user_fis_number: NotRequired[Nullable[str]]
    r"""Customer’s document type"""
    fiscal_number: NotRequired[Nullable[str]]
    r"""Customer’s document number"""
    bank_code: NotRequired[Nullable[str]]
    r"""The bank code of the selected bank"""


class NuveiPSEOptions(BaseModel):
    user_type: Annotated[OptionalNullable[str], pydantic.Field(alias="userType")] = (
        UNSET
    )
    r"""Customer type (\"N\" for persona natural, \"J\" for persona jurídica)"""

    user_fis_number: Annotated[
        OptionalNullable[str], pydantic.Field(alias="userFisNumber")
    ] = UNSET
    r"""Customer’s document type"""

    fiscal_number: Annotated[
        OptionalNullable[str], pydantic.Field(alias="fiscalNumber")
    ] = UNSET
    r"""Customer’s document number"""

    bank_code: Annotated[OptionalNullable[str], pydantic.Field(alias="bankCode")] = (
        UNSET
    )
    r"""The bank code of the selected bank"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["userType", "userFisNumber", "fiscalNumber", "bankCode"])
        nullable_fields = set(["userType", "userFisNumber", "fiscalNumber", "bankCode"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            is_nullable_and_explicitly_set = (
                k in nullable_fields
                and (self.__pydantic_fields_set__.intersection({n}))  # pylint: disable=no-member
            )

            if val != UNSET_SENTINEL:
                if (
                    val is not None
                    or k not in optional_fields
                    or is_nullable_and_explicitly_set
                ):
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .forterantifraudoptionscartitem import (
    ForterAntiFraudOptionsCartItem,
    ForterAntiFraudOptionsCartItemTypedDict,
)
from .forterantifraudoptionsdiscount import (
    ForterAntiFraudOptionsDiscount,
    ForterAntiFraudOptionsDiscountTypedDict,
)
from gr4vy.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
    UnrecognizedStr,
)
from pydantic import model_serializer
from typing import List, Literal, Union
from typing_extensions import NotRequired, TypedDict


ForterAntiFraudOptionsDeliveryType = Union[
    Literal[
        "DIGITAL",
        "PHYSICAL",
        "HYBRID",
    ],
    UnrecognizedStr,
]


class ForterAntiFraudOptionsTypedDict(TypedDict):
    delivery_type: NotRequired[Nullable[ForterAntiFraudOptionsDeliveryType]]
    r"""The delivery type"""
    delivery_method: NotRequired[Nullable[str]]
    r"""The delivery method"""
    is_guest_buyer: NotRequired[Nullable[bool]]
    r"""Defines if this payment is made using guest checkout."""
    cart_items: NotRequired[Nullable[List[ForterAntiFraudOptionsCartItemTypedDict]]]
    r"""A list of cart items details to pass to the Forter API."""
    total_discount: NotRequired[Nullable[ForterAntiFraudOptionsDiscountTypedDict]]
    r"""Information about the discount applied to this order."""


class ForterAntiFraudOptions(BaseModel):
    delivery_type: OptionalNullable[ForterAntiFraudOptionsDeliveryType] = UNSET
    r"""The delivery type"""

    delivery_method: OptionalNullable[str] = UNSET
    r"""The delivery method"""

    is_guest_buyer: OptionalNullable[bool] = UNSET
    r"""Defines if this payment is made using guest checkout."""

    cart_items: OptionalNullable[List[ForterAntiFraudOptionsCartItem]] = UNSET
    r"""A list of cart items details to pass to the Forter API."""

    total_discount: OptionalNullable[ForterAntiFraudOptionsDiscount] = UNSET
    r"""Information about the discount applied to this order."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "delivery_type",
                "delivery_method",
                "is_guest_buyer",
                "cart_items",
                "total_discount",
            ]
        )
        nullable_fields = set(
            [
                "delivery_type",
                "delivery_method",
                "is_guest_buyer",
                "cart_items",
                "total_discount",
            ]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            is_nullable_and_explicitly_set = (
                k in nullable_fields
                and (self.__pydantic_fields_set__.intersection({n}))  # pylint: disable=no-member
            )

            if val != UNSET_SENTINEL:
                if (
                    val is not None
                    or k not in optional_fields
                    or is_nullable_and_explicitly_set
                ):
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .transaction import Transaction, TransactionTypedDict
from .transactionvoid import TransactionVoid, TransactionVoidTypedDict
from gr4vy.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from gr4vy.utils import FieldMetadata, HeaderMetadata, PathParamMetadata
import pydantic
from pydantic import model_serializer
from typing import List, Optional, Union
from typing_extensions import Annotated, NotRequired, TypeAliasType, TypedDict


class VoidTransactionGlobalsTypedDict(TypedDict):
    merchant_account_id: NotRequired[str]
    r"""The ID of the merchant account to use for this request."""


class VoidTransactionGlobals(BaseModel):
    merchant_account_id: Annotated[
        Optional[str],
        pydantic.Field(alias="x-gr4vy-merchant-account-id"),
        FieldMetadata(header=HeaderMetadata(style="simple", explode=False)),
    ] = None
    r"""The ID of the merchant account to use for this request."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["merchant_account_id"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class VoidTransactionRequestTypedDict(TypedDict):
    transaction_id: str
    r"""The ID of the transaction"""
    prefer: NotRequired[Nullable[List[str]]]
    r"""The preferred resource type in the response."""
    merchant_account_id: NotRequired[str]
    r"""The ID of the merchant account to use for this request."""


class VoidTransactionRequest(BaseModel):
    transaction_id: Annotated[
        str, FieldMetadata(path=PathParamMetadata(style="simple", explode=False))
    ]
    r"""The ID of the transaction"""

    prefer: Annotated[
        OptionalNullable[List[str]],
        FieldMetadata(header=HeaderMetadata(style="simple", explode=False)),
    ] = UNSET
    r"""The preferred resource type in the response."""

    merchant_account_id: Annotated[
        Optional[str],
        pydantic.Field(alias="x-gr4vy-merchant-account-id"),
        FieldMetadata(header=HeaderMetadata(style="simple", explode=False)),
    ] = None
    r"""The ID of the merchant account to use for this request."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["prefer", "merchant_account_id"])
        nullable_fields = set(["prefer"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            is_nullable_and_explicitly_set = (
                k in nullable_fields
                and (self.__pydantic_fields_set__.intersection({n}))  # pylint: disable=no-member
            )

            if val != UNSET_SENTINEL:
                if (
                    val is not None
                    or k not in optional_fields
                    or is_nullable_and_explicitly_set
                ):
                    m[k] = val

        return m


ResponseVoidTransactionTypedDict = TypeAliasType(
    "ResponseVoidTransactionTypedDict",
    Union[TransactionVoidTypedDict, TransactionTypedDict],
)
r"""Successful Response"""


ResponseVoidTransaction = TypeAliasType(
    "ResponseVoidTransaction", Union[TransactionVoid, Transaction]
)
r"""Successful Response"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .field import FieldT, FieldTTypedDict
from .merchantprofilescheme import MerchantProfileScheme, MerchantProfileSchemeTypedDict
from gr4vy.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Dict, List, Optional
from typing_extensions import NotRequired, TypedDict


class PaymentServiceCreateTypedDict(TypedDict):
    r"""Request body for activating a payment service"""

    display_name: str
    r"""The display name for the payment service."""
    payment_service_definition_id: str
    r"""The definition ID of the service to configure."""
    fields: List[FieldTTypedDict]
    r"""The non-secret credential fields that have been configured for this payment service. Any secret fields are omitted."""
    accepted_currencies: List[str]
    r"""A list of currencies for which this service is enabled, in ISO 4217 three-letter code format."""
    accepted_countries: List[str]
    r"""A list of countries for which this service is enabled, in ISO two-letter code format."""
    reporting_fields: NotRequired[Nullable[List[FieldTTypedDict]]]
    r"""The non-secret reporting fields that have been configured for this payment service. Any secret fields are omitted."""
    position: NotRequired[Nullable[int]]
    r"""Deprecated field used to define the order in which to process payment services"""
    active: NotRequired[Nullable[bool]]
    r"""Defines if this payment service is currently active."""
    three_d_secure_enabled: NotRequired[bool]
    r"""Defines if this payment service has 3DS enabled."""
    merchant_profile: NotRequired[
        Nullable[Dict[str, Nullable[MerchantProfileSchemeTypedDict]]]
    ]
    r"""An object containing a key for each supported card schemes, and for each key an object with the 3DS profile for this service for that scheme."""
    payment_method_tokenization_enabled: NotRequired[Nullable[bool]]
    r"""Defines if this payment service support payment method tokenization."""
    network_tokens_enabled: NotRequired[Nullable[bool]]
    r"""Defines if this payment service supports network tokens."""
    open_loop: NotRequired[Nullable[bool]]
    r"""Defines if this payment service is open loop."""
    settlement_reporting_enabled: NotRequired[bool]
    r"""Defines if this payment service has settlement reporting enabled."""


class PaymentServiceCreate(BaseModel):
    r"""Request body for activating a payment service"""

    display_name: str
    r"""The display name for the payment service."""

    payment_service_definition_id: str
    r"""The definition ID of the service to configure."""

    fields: List[FieldT]
    r"""The non-secret credential fields that have been configured for this payment service. Any secret fields are omitted."""

    accepted_currencies: List[str]
    r"""A list of currencies for which this service is enabled, in ISO 4217 three-letter code format."""

    accepted_countries: List[str]
    r"""A list of countries for which this service is enabled, in ISO two-letter code format."""

    reporting_fields: OptionalNullable[List[FieldT]] = UNSET
    r"""The non-secret reporting fields that have been configured for this payment service. Any secret fields are omitted."""

    position: OptionalNullable[int] = UNSET
    r"""Deprecated field used to define the order in which to process payment services"""

    active: OptionalNullable[bool] = UNSET
    r"""Defines if this payment service is currently active."""

    three_d_secure_enabled: Optional[bool] = False
    r"""Defines if this payment service has 3DS enabled."""

    merchant_profile: OptionalNullable[Dict[str, Nullable[MerchantProfileScheme]]] = (
        UNSET
    )
    r"""An object containing a key for each supported card schemes, and for each key an object with the 3DS profile for this service for that scheme."""

    payment_method_tokenization_enabled: OptionalNullable[bool] = UNSET
    r"""Defines if this payment service support payment method tokenization."""

    network_tokens_enabled: OptionalNullable[bool] = UNSET
    r"""Defines if this payment service supports network tokens."""

    open_loop: OptionalNullable[bool] = UNSET
    r"""Defines if this payment service is open loop."""

    settlement_reporting_enabled: Optional[bool] = False
    r"""Defines if this payment service has settlement reporting enabled."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "reporting_fields",
                "position",
                "active",
                "three_d_secure_enabled",
                "merchant_profile",
                "payment_method_tokenization_enabled",
                "network_tokens_enabled",
                "open_loop",
                "settlement_reporting_enabled",
            ]
        )
        nullable_fields = set(
            [
                "reporting_fields",
                "position",
                "active",
                "merchant_profile",
                "payment_method_tokenization_enabled",
                "network_tokens_enabled",
                "open_loop",
            ]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            is_nullable_and_explicitly_set = (
                k in nullable_fields
                and (self.__pydantic_fields_set__.intersection({n}))  # pylint: disable=no-member
            )

            if val != UNSET_SENTINEL:
                if (
                    val is not None
                    or k not in optional_fields
                    or is_nullable_and_explicitly_set
                ):
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .networktokenstatus import NetworkTokenStatus
from datetime import datetime
from gr4vy.types import BaseModel, UNSET_SENTINEL
from gr4vy.utils import validate_const
import pydantic
from pydantic import model_serializer
from pydantic.functional_validators import AfterValidator
from typing import Literal, Optional
from typing_extensions import Annotated, TypedDict


class NetworkTokenTypedDict(TypedDict):
    id: str
    r"""The ID for the network token."""
    expiration_date: str
    r"""The expiration date for the network token."""
    payment_method_id: str
    r"""The ID of the payment method used to generate this token"""
    status: NetworkTokenStatus
    token: str
    r"""The token value. Will be present if succeeded."""
    created_at: datetime
    r"""The date and time when this network token was first created in our system."""
    updated_at: datetime
    r"""The date and time when this network token was last updated in our system."""
    type: Literal["network-token"]
    r"""Always `network-token`."""


class NetworkToken(BaseModel):
    id: str
    r"""The ID for the network token."""

    expiration_date: str
    r"""The expiration date for the network token."""

    payment_method_id: str
    r"""The ID of the payment method used to generate this token"""

    status: NetworkTokenStatus

    token: str
    r"""The token value. Will be present if succeeded."""

    created_at: datetime
    r"""The date and time when this network token was first created in our system."""

    updated_at: datetime
    r"""The date and time when this network token was last updated in our system."""

    TYPE: Annotated[
        Annotated[
            Optional[Literal["network-token"]],
            AfterValidator(validate_const("network-token")),
        ],
        pydantic.Field(alias="type"),
    ] = "network-token"
    r"""Always `network-token`."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["type"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .userstatus import UserStatus
from gr4vy.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from gr4vy.utils import validate_const
import pydantic
from pydantic import model_serializer
from pydantic.functional_validators import AfterValidator
from typing import Literal, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class AuditLogEntryUserTypedDict(TypedDict):
    name: str
    r"""The name of the user."""
    is_staff: bool
    r"""Whether this is a Gr4vy staff user."""
    status: UserStatus
    type: Literal["user"]
    r"""Always `user`."""
    id: NotRequired[Nullable[str]]
    r"""The ID of the user."""
    email_address: NotRequired[Nullable[str]]
    r"""The email address for this user."""


class AuditLogEntryUser(BaseModel):
    name: str
    r"""The name of the user."""

    is_staff: bool
    r"""Whether this is a Gr4vy staff user."""

    status: UserStatus

    TYPE: Annotated[
        Annotated[Optional[Literal["user"]], AfterValidator(validate_const("user"))],
        pydantic.Field(alias="type"),
    ] = "user"
    r"""Always `user`."""

    id: OptionalNullable[str] = UNSET
    r"""The ID of the user."""

    email_address: OptionalNullable[str] = UNSET
    r"""The email address for this user."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["type", "id", "email_address"])
        nullable_fields = set(["id", "email_address"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            is_nullable_and_explicitly_set = (
                k in nullable_fields
                and (self.__pydantic_fields_set__.intersection({n}))  # pylint: disable=no-member
            )

            if val != UNSET_SENTINEL:
                if (
                    val is not None
                    or k not in optional_fields
                    or is_nullable_and_explicitly_set
                ):
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from gr4vy.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
    UnrecognizedStr,
)
from pydantic import model_serializer
from typing import Literal, Union
from typing_extensions import NotRequired, TypedDict


VaultPaymentMethodCriteria = Union[
    Literal[
        "ALWAYS",
        "ON_SUCCESSFUL_TRANSACTION",
    ],
    UnrecognizedStr,
]


class BraintreeDynamicDataFieldsOptionsTypedDict(TypedDict):
    three_ds_auth_status: NotRequired[Nullable[str]]
    r"""Passes the 3DS status to the Braintree API using `customFields` with the key set to the value of `three_ds_auth_status`"""
    purchase_order_number: NotRequired[Nullable[str]]
    r"""Passes the `transaction.purchaseOrderNumber` field when creating a new transaction."""
    vault_payment_method_criteria: NotRequired[Nullable[VaultPaymentMethodCriteria]]
    r"""Passes the `vaultPaymentMethodCriteria` field when creating a new transaction."""


class BraintreeDynamicDataFieldsOptions(BaseModel):
    three_ds_auth_status: OptionalNullable[str] = UNSET
    r"""Passes the 3DS status to the Braintree API using `customFields` with the key set to the value of `three_ds_auth_status`"""

    purchase_order_number: OptionalNullable[str] = UNSET
    r"""Passes the `transaction.purchaseOrderNumber` field when creating a new transaction."""

    vault_payment_method_criteria: OptionalNullable[VaultPaymentMethodCriteria] = UNSET
    r"""Passes the `vaultPaymentMethodCriteria` field when creating a new transaction."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "three_ds_auth_status",
                "purchase_order_number",
                "vault_payment_method_criteria",
            ]
        )
        nullable_fields = set(
            [
                "three_ds_auth_status",
                "purchase_order_number",
                "vault_payment_method_criteria",
            ]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            is_nullable_and_explicitly_set = (
                k in nullable_fields
                and (self.__pydantic_fields_set__.intersection({n}))  # pylint: disable=no-member
            )

            if val != UNSET_SENTINEL:
                if (
                    val is not None
                    or k not in optional_fields
                    or is_nullable_and_explicitly_set
                ):
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .refundtargettype import RefundTargetType
from gr4vy.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class TransactionRefundCreateTypedDict(TypedDict):
    amount: NotRequired[Nullable[int]]
    r"""The amount to refund, in the smallest currency unit (e.g., cents). If omitted, a full refund will be requested."""
    target_type: NotRequired[RefundTargetType]
    target_id: NotRequired[Nullable[str]]
    r"""The optional ID of the instrument to refund for. This is only required when the `target_type` is set to `gift-card-redemption`."""
    reason: NotRequired[Nullable[str]]
    r"""An optional reason to attach extra context to the refund request."""
    external_identifier: NotRequired[Nullable[str]]
    r"""An external identifier that can be used to match the refund against your own records."""


class TransactionRefundCreate(BaseModel):
    amount: OptionalNullable[int] = UNSET
    r"""The amount to refund, in the smallest currency unit (e.g., cents). If omitted, a full refund will be requested."""

    target_type: Optional[RefundTargetType] = None

    target_id: OptionalNullable[str] = UNSET
    r"""The optional ID of the instrument to refund for. This is only required when the `target_type` is set to `gift-card-redemption`."""

    reason: OptionalNullable[str] = UNSET
    r"""An optional reason to attach extra context to the refund request."""

    external_identifier: OptionalNullable[str] = UNSET
    r"""An external identifier that can be used to match the refund against your own records."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            ["amount", "target_type", "target_id", "reason", "external_identifier"]
        )
        nullable_fields = set(["amount", "target_id", "reason", "external_identifier"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            is_nullable_and_explicitly_set = (
                k in nullable_fields
                and (self.__pydantic_fields_set__.intersection({n}))  # pylint: disable=no-member
            )

            if val != UNSET_SENTINEL:
                if (
                    val is not None
                    or k not in optional_fields
                    or is_nullable_and_explicitly_set
                ):
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .threedsecurescenarioconditions import (
    ThreeDSecureScenarioConditions,
    ThreeDSecureScenarioConditionsTypedDict,
)
from .threedsecurescenariooutcome import (
    ThreeDSecureScenarioOutcome,
    ThreeDSecureScenarioOutcomeTypedDict,
)
from datetime import datetime
from gr4vy.types import BaseModel, UNSET_SENTINEL
from gr4vy.utils import validate_const
import pydantic
from pydantic import model_serializer
from pydantic.functional_validators import AfterValidator
from typing import Literal, Optional
from typing_extensions import Annotated, TypedDict


class ThreeDSecureScenarioTypedDict(TypedDict):
    id: str
    r"""Unique identifier for the 3DS scenario"""
    merchant_account_id: str
    r"""ID of the associated merchant account"""
    created_at: datetime
    r"""The date and time when this 3DS scenario was first created in our system."""
    updated_at: datetime
    r"""The date and time when this 3DS scenario was last updated in our system."""
    conditions: ThreeDSecureScenarioConditionsTypedDict
    outcome: ThreeDSecureScenarioOutcomeTypedDict
    type: Literal["three-d-secure-scenario"]
    r"""Always `three-d-secure-scenario`."""


class ThreeDSecureScenario(BaseModel):
    id: str
    r"""Unique identifier for the 3DS scenario"""

    merchant_account_id: str
    r"""ID of the associated merchant account"""

    created_at: datetime
    r"""The date and time when this 3DS scenario was first created in our system."""

    updated_at: datetime
    r"""The date and time when this 3DS scenario was last updated in our system."""

    conditions: ThreeDSecureScenarioConditions

    outcome: ThreeDSecureScenarioOutcome

    TYPE: Annotated[
        Annotated[
            Optional[Literal["three-d-secure-scenario"]],
            AfterValidator(validate_const("three-d-secure-scenario")),
        ],
        pydantic.Field(alias="type"),
    ] = "three-d-secure-scenario"
    r"""Always `three-d-secure-scenario`."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["type"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from gr4vy.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing_extensions import Annotated, NotRequired, TypedDict


class TrustlyOptionsTypedDict(TypedDict):
    refresh_split_token: NotRequired[Nullable[bool]]
    r"""Indicates to Gr4vy whether or not the stored Trustly agreement needs refreshing."""


class TrustlyOptions(BaseModel):
    refresh_split_token: Annotated[
        OptionalNullable[bool], pydantic.Field(alias="refreshSplitToken")
    ] = UNSET
    r"""Indicates to Gr4vy whether or not the stored Trustly agreement needs refreshing."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["refreshSplitToken"])
        nullable_fields = set(["refreshSplitToken"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            is_nullable_and_explicitly_set = (
                k in nullable_fields
                and (self.__pydantic_fields_set__.intersection({n}))  # pylint: disable=no-member
            )

            if val != UNSET_SENTINEL:
                if (
                    val is not None
                    or k not in optional_fields
                    or is_nullable_and_explicitly_set
                ):
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .paymentservice import PaymentService, PaymentServiceTypedDict
from gr4vy.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from pydantic import model_serializer
from typing import List, Optional
from typing_extensions import NotRequired, TypedDict


class PaymentServicesTypedDict(TypedDict):
    items: List[PaymentServiceTypedDict]
    r"""A list of items returned for this request."""
    limit: NotRequired[int]
    r"""The number of items for this page."""
    next_cursor: NotRequired[Nullable[str]]
    r"""The cursor pointing at the next page of items."""
    previous_cursor: NotRequired[Nullable[str]]
    r"""The cursor pointing at the previous page of items."""


class PaymentServices(BaseModel):
    items: List[PaymentService]
    r"""A list of items returned for this request."""

    limit: Optional[int] = 20
    r"""The number of items for this page."""

    next_cursor: OptionalNullable[str] = UNSET
    r"""The cursor pointing at the next page of items."""

    previous_cursor: OptionalNullable[str] = UNSET
    r"""The cursor pointing at the previous page of items."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["limit", "next_cursor", "previous_cursor"])
        nullable_fields = set(["next_cursor", "previous_cursor"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            is_nullable_and_explicitly_set = (
                k in nullable_fields
                and (self.__pydantic_fields_set__.intersection({n}))  # pylint: disable=no-member
            )

            if val != UNSET_SENTINEL:
                if (
                    val is not None
                    or k not in optional_fields
                    or is_nullable_and_explicitly_set
                ):
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .cardscheme import CardScheme
from gr4vy.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from pydantic import model_serializer
from typing_extensions import NotRequired, TypedDict


class ThreeDSecureDataV1TypedDict(TypedDict):
    cavv: str
    r"""The cardholder authentication value or AAV."""
    eci: str
    r"""The ecommerce indicator for the 3DS transaction."""
    version: str
    r"""The version of 3-D Secure that was used."""
    directory_response: str
    r"""For 3-D Secure version 1, the enrolment response. For 3-D Secure version 2 and above, the transaction status from the `ARes`."""
    authentication_response: str
    r"""The response for the 3DS authentication call."""
    cavv_algorithm: str
    r"""The CAVV algorithm used."""
    xid: str
    r"""The transaction identifier."""
    scheme: NotRequired[Nullable[CardScheme]]
    r"""The scheme/brand of the card that is used for 3-D Secure."""


class ThreeDSecureDataV1(BaseModel):
    cavv: str
    r"""The cardholder authentication value or AAV."""

    eci: str
    r"""The ecommerce indicator for the 3DS transaction."""

    version: str
    r"""The version of 3-D Secure that was used."""

    directory_response: str
    r"""For 3-D Secure version 1, the enrolment response. For 3-D Secure version 2 and above, the transaction status from the `ARes`."""

    authentication_response: str
    r"""The response for the 3DS authentication call."""

    cavv_algorithm: str
    r"""The CAVV algorithm used."""

    xid: str
    r"""The transaction identifier."""

    scheme: OptionalNullable[CardScheme] = UNSET
    r"""The scheme/brand of the card that is used for 3-D Secure."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["scheme"])
        nullable_fields = set(["scheme"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            is_nullable_and_explicitly_set = (
                k in nullable_fields
                and (self.__pydantic_fields_set__.intersection({n}))  # pylint: disable=no-member
            )

            if val != UNSET_SENTINEL:
                if (
                    val is not None
                    or k not in optional_fields
                    or is_nullable_and_explicitly_set
                ):
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .airline import Airline, AirlineTypedDict
from gr4vy.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from pydantic import model_serializer
from typing_extensions import NotRequired, TypedDict


class TransactionCaptureCreateTypedDict(TypedDict):
    r"""Request body for capturing an authorized transaction."""

    amount: NotRequired[Nullable[int]]
    r"""The amount to capture, in the smallest currency unit (e.g., cents). This must be less than or equal to the authorized amount, unless over-capture is available."""
    airline: NotRequired[Nullable[AirlineTypedDict]]
    r"""The airline data to submit to the payment service during the capture call."""


class TransactionCaptureCreate(BaseModel):
    r"""Request body for capturing an authorized transaction."""

    amount: OptionalNullable[int] = UNSET
    r"""The amount to capture, in the smallest currency unit (e.g., cents). This must be less than or equal to the authorized amount, unless over-capture is available."""

    airline: OptionalNullable[Airline] = UNSET
    r"""The airline data to submit to the payment service during the capture call."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["amount", "airline"])
        nullable_fields = set(["amount", "airline"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            is_nullable_and_explicitly_set = (
                k in nullable_fields
                and (self.__pydantic_fields_set__.intersection({n}))  # pylint: disable=no-member
            )

            if val != UNSET_SENTINEL:
                if (
                    val is not None
                    or k not in optional_fields
                    or is_nullable_and_explicitly_set
                ):
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .digitalwalletprovider import DigitalWalletProvider
from datetime import datetime
from gr4vy.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from gr4vy.utils import validate_const
import pydantic
from pydantic import model_serializer
from pydantic.functional_validators import AfterValidator
from typing import Any, Dict, List, Literal, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class DigitalWalletTypedDict(TypedDict):
    id: str
    r"""The ID for the digital wallet."""
    merchant_account_id: str
    r"""The ID of the merchant account this digital wallet belongs to."""
    provider: DigitalWalletProvider
    merchant_name: str
    r"""The name of the merchant the digital wallet is registered to."""
    domain_names: List[str]
    r"""The list of domain names that a digital wallet can be used on (deprecated)."""
    created_at: datetime
    r"""The date this buyer was created at."""
    updated_at: datetime
    r"""The date this buyer was last updated at."""
    type: Literal["digital-wallet"]
    r"""Always `digital-wallet`."""
    merchant_display_name: NotRequired[Nullable[str]]
    r"""The consumer facing name of the merchant."""
    merchant_url: NotRequired[Nullable[str]]
    r"""The main URL of the merchant."""
    merchant_country_code: NotRequired[Nullable[str]]
    r"""The country code where the merchant is registered."""
    active_certificate_count: NotRequired[int]
    r"""The number of active custom certificates registered for this digital wallet (Apple Pay only)."""
    pending_certificate_count: NotRequired[int]
    r"""The number of pending custom certificates registered for this digital wallet (Apple Pay only)."""
    expired_certificate_count: NotRequired[int]
    r"""The number of expired custom certificates registered for this digital wallet (Apple Pay only)."""
    fields: NotRequired[Nullable[Dict[str, Any]]]
    r"""Custom attributes for some digital wallets. Currently only used by Click to Pay."""


class DigitalWallet(BaseModel):
    id: str
    r"""The ID for the digital wallet."""

    merchant_account_id: str
    r"""The ID of the merchant account this digital wallet belongs to."""

    provider: DigitalWalletProvider

    merchant_name: str
    r"""The name of the merchant the digital wallet is registered to."""

    domain_names: List[str]
    r"""The list of domain names that a digital wallet can be used on (deprecated)."""

    created_at: datetime
    r"""The date this buyer was created at."""

    updated_at: datetime
    r"""The date this buyer was last updated at."""

    TYPE: Annotated[
        Annotated[
            Optional[Literal["digital-wallet"]],
            AfterValidator(validate_const("digital-wallet")),
        ],
        pydantic.Field(alias="type"),
    ] = "digital-wallet"
    r"""Always `digital-wallet`."""

    merchant_display_name: OptionalNullable[str] = UNSET
    r"""The consumer facing name of the merchant."""

    merchant_url: OptionalNullable[str] = UNSET
    r"""The main URL of the merchant."""

    merchant_country_code: OptionalNullable[str] = UNSET
    r"""The country code where the merchant is registered."""

    active_certificate_count: Optional[int] = 0
    r"""The number of active custom certificates registered for this digital wallet (Apple Pay only)."""

    pending_certificate_count: Optional[int] = 0
    r"""The number of pending custom certificates registered for this digital wallet (Apple Pay only)."""

    expired_certificate_count: Optional[int] = 0
    r"""The number of expired custom certificates registered for this digital wallet (Apple Pay only)."""

    fields: OptionalNullable[Dict[str, Any]] = UNSET
    r"""Custom attributes for some digital wallets. Currently only used by Click to Pay."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "type",
                "merchant_display_name",
                "merchant_url",
                "merchant_country_code",
                "active_certificate_count",
                "pending_certificate_count",
                "expired_certificate_count",
                "fields",
            ]
        )
        nullable_fields = set(
            ["merchant_display_name", "merchant_url", "merchant_country_code", "fields"]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            is_nullable_and_explicitly_set = (
                k in nullable_fields
                and (self.__pydantic_fields_set__.intersection({n}))  # pylint: disable=no-member
            )

            if val != UNSET_SENTINEL:
                if (
                    val is not None
                    or k not in optional_fields
                    or is_nullable_and_explicitly_set
                ):
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from gr4vy.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import Dict
from typing_extensions import Annotated, NotRequired, TypedDict


class WpayEverdaypayOptionsTypedDict(TypedDict):
    merchant_defined_data: NotRequired[Nullable[Dict[str, str]]]
    r"""A dictionary of merchant defined data, to be passed to Wpay for anti-fraud control."""
    customer_id: NotRequired[Nullable[str]]
    r"""The customer ID for the Everyday Rewards account."""
    rewards_access_token: NotRequired[Nullable[str]]
    r"""The access token for the Everyday Rewards account."""
    device_id: NotRequired[Nullable[str]]
    r"""The ID of the device on which the payment is occuring."""
    post_payment_redirect: NotRequired[Nullable[bool]]
    r"""Whether the transaction should redirect post-payment"""


class WpayEverdaypayOptions(BaseModel):
    merchant_defined_data: OptionalNullable[Dict[str, str]] = UNSET
    r"""A dictionary of merchant defined data, to be passed to Wpay for anti-fraud control."""

    customer_id: Annotated[
        OptionalNullable[str], pydantic.Field(alias="customerId")
    ] = UNSET
    r"""The customer ID for the Everyday Rewards account."""

    rewards_access_token: Annotated[
        OptionalNullable[str], pydantic.Field(alias="rewardsAccessToken")
    ] = UNSET
    r"""The access token for the Everyday Rewards account."""

    device_id: Annotated[OptionalNullable[str], pydantic.Field(alias="deviceId")] = (
        UNSET
    )
    r"""The ID of the device on which the payment is occuring."""

    post_payment_redirect: Annotated[
        OptionalNullable[bool], pydantic.Field(alias="postPaymentRedirect")
    ] = UNSET
    r"""Whether the transaction should redirect post-payment"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "merchant_defined_data",
                "customerId",
                "rewardsAccessToken",
                "deviceId",
                "postPaymentRedirect",
            ]
        )
        nullable_fields = set(
            [
                "merchant_defined_data",
                "customerId",
                "rewardsAccessToken",
                "deviceId",
                "postPaymentRedirect",
            ]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            is_nullable_and_explicitly_set = (
                k in nullable_fields
                and (self.__pydantic_fields_set__.intersection({n}))  # pylint: disable=no-member
            )

            if val != UNSET_SENTINEL:
                if (
                    val is not None
                    or k not in optional_fields
                    or is_nullable_and_explicitly_set
                ):
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from gr4vy.types import BaseModel, UnrecognizedStr
from typing import Literal, Union
from typing_extensions import TypedDict


ThreeDSecureScenarioOutcomeResultTransactionStatus = Union[
    Literal[
        "Y",
        "N",
        "timeout",
    ],
    UnrecognizedStr,
]
r"""3DS result."""


class ThreeDSecureScenarioOutcomeResultTypedDict(TypedDict):
    transaction_status: ThreeDSecureScenarioOutcomeResultTransactionStatus
    r"""3DS result."""


class ThreeDSecureScenarioOutcomeResult(BaseModel):
    transaction_status: ThreeDSecureScenarioOutcomeResultTransactionStatus
    r"""3DS result."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .cartitem import CartItem, CartItemTypedDict
from .guestbuyer import GuestBuyer, GuestBuyerTypedDict
from .statementdescriptor import StatementDescriptor, StatementDescriptorTypedDict
from .transactionconnectionoptions import (
    TransactionConnectionOptions,
    TransactionConnectionOptionsTypedDict,
)
from .transactionintent import TransactionIntent
from .transactionpaymentsource import TransactionPaymentSource
from datetime import datetime
from gr4vy.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Any, Dict, List, Optional
from typing_extensions import NotRequired, TypedDict


class PaymentLinkCreateTypedDict(TypedDict):
    amount: int
    r"""The amount for the payment link."""
    country: str
    r"""The country code for the payment link."""
    currency: str
    r"""The currency code for the payment link."""
    buyer: NotRequired[Nullable[GuestBuyerTypedDict]]
    r"""The guest buyer for the payment link."""
    expires_at: NotRequired[Nullable[datetime]]
    r"""The expiration date and time for the payment link."""
    connection_options: NotRequired[Nullable[TransactionConnectionOptionsTypedDict]]
    r"""Connection options for the payment link."""
    external_identifier: NotRequired[Nullable[str]]
    r"""The merchant reference for the payment link."""
    statement_descriptor: NotRequired[Nullable[StatementDescriptorTypedDict]]
    r"""The statement descriptor for the payment link."""
    locale: NotRequired[Nullable[str]]
    r"""The locale for the payment link."""
    merchant_name: NotRequired[Nullable[str]]
    r"""The merchant's display name."""
    merchant_url: NotRequired[Nullable[str]]
    r"""The merchant's website URL."""
    merchant_banner_url: NotRequired[Nullable[str]]
    r"""The merchant's banner image URL."""
    merchant_color: NotRequired[Nullable[str]]
    r"""The merchant's brand color."""
    merchant_message: NotRequired[Nullable[str]]
    r"""A message from the merchant."""
    merchant_terms_and_conditions_url: NotRequired[Nullable[str]]
    r"""URL to the merchant's terms and conditions."""
    merchant_favicon_url: NotRequired[Nullable[str]]
    r"""URL to the merchant's favicon."""
    intent: NotRequired[TransactionIntent]
    return_url: NotRequired[Nullable[str]]
    r"""The return URL after payment completion."""
    cart_items: NotRequired[Nullable[List[CartItemTypedDict]]]
    r"""The cart items for the payment link."""
    metadata: NotRequired[Nullable[Dict[str, Any]]]
    r"""Arbitrary metadata for the payment link."""
    payment_source: NotRequired[TransactionPaymentSource]
    r"""The way payment method information made it to this transaction."""
    store: NotRequired[bool]
    r"""Whether to store the payment method for future use."""
    buyer_id: NotRequired[Nullable[str]]
    r"""The ID of the buyer to associate the payment method with. Note: When `buyer_id` is provided, the payment link should be treated as a secret as it will allow the user to manage payment methods for the associated buyer."""
    installment_count: NotRequired[Nullable[int]]
    r"""The number of installments a buyer is required to make."""


class PaymentLinkCreate(BaseModel):
    amount: int
    r"""The amount for the payment link."""

    country: str
    r"""The country code for the payment link."""

    currency: str
    r"""The currency code for the payment link."""

    buyer: OptionalNullable[GuestBuyer] = UNSET
    r"""The guest buyer for the payment link."""

    expires_at: OptionalNullable[datetime] = UNSET
    r"""The expiration date and time for the payment link."""

    connection_options: OptionalNullable[TransactionConnectionOptions] = UNSET
    r"""Connection options for the payment link."""

    external_identifier: OptionalNullable[str] = UNSET
    r"""The merchant reference for the payment link."""

    statement_descriptor: OptionalNullable[StatementDescriptor] = UNSET
    r"""The statement descriptor for the payment link."""

    locale: OptionalNullable[str] = UNSET
    r"""The locale for the payment link."""

    merchant_name: OptionalNullable[str] = UNSET
    r"""The merchant's display name."""

    merchant_url: OptionalNullable[str] = UNSET
    r"""The merchant's website URL."""

    merchant_banner_url: OptionalNullable[str] = UNSET
    r"""The merchant's banner image URL."""

    merchant_color: OptionalNullable[str] = UNSET
    r"""The merchant's brand color."""

    merchant_message: OptionalNullable[str] = UNSET
    r"""A message from the merchant."""

    merchant_terms_and_conditions_url: OptionalNullable[str] = UNSET
    r"""URL to the merchant's terms and conditions."""

    merchant_favicon_url: OptionalNullable[str] = UNSET
    r"""URL to the merchant's favicon."""

    intent: Optional[TransactionIntent] = None

    return_url: OptionalNullable[str] = UNSET
    r"""The return URL after payment completion."""

    cart_items: OptionalNullable[List[CartItem]] = UNSET
    r"""The cart items for the payment link."""

    metadata: OptionalNullable[Dict[str, Any]] = UNSET
    r"""Arbitrary metadata for the payment link."""

    payment_source: Optional[TransactionPaymentSource] = None
    r"""The way payment method information made it to this transaction."""

    store: Optional[bool] = False
    r"""Whether to store the payment method for future use."""

    buyer_id: OptionalNullable[str] = UNSET
    r"""The ID of the buyer to associate the payment method with. Note: When `buyer_id` is provided, the payment link should be treated as a secret as it will allow the user to manage payment methods for the associated buyer."""

    installment_count: OptionalNullable[int] = UNSET
    r"""The number of installments a buyer is required to make."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "buyer",
                "expires_at",
                "connection_options",
                "external_identifier",
                "statement_descriptor",
                "locale",
                "merchant_name",
                "merchant_url",
                "merchant_banner_url",
                "merchant_color",
                "merchant_message",
                "merchant_terms_and_conditions_url",
                "merchant_favicon_url",
                "intent",
                "return_url",
                "cart_items",
                "metadata",
                "payment_source",
                "store",
                "buyer_id",
                "installment_count",
            ]
        )
        nullable_fields = set(
            [
                "buyer",
                "expires_at",
                "connection_options",
                "external_identifier",
                "statement_descriptor",
                "locale",
                "merchant_name",
                "merchant_url",
                "merchant_banner_url",
                "merchant_color",
                "merchant_message",
                "merchant_terms_and_conditions_url",
                "merchant_favicon_url",
                "return_url",
                "cart_items",
                "metadata",
                "buyer_id",
                "installment_count",
            ]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            is_nullable_and_explicitly_set = (
                k in nullable_fields
                and (self.__pydantic_fields_set__.intersection({n}))  # pylint: disable=no-member
            )

            if val != UNSET_SENTINEL:
                if (
                    val is not None
                    or k not in optional_fields
                    or is_nullable_and_explicitly_set
                ):
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .payoutsummaries import PayoutSummaries, PayoutSummariesTypedDict
from gr4vy.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from gr4vy.utils import FieldMetadata, HeaderMetadata, QueryParamMetadata
import pydantic
from pydantic import model_serializer
from typing import Callable, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class ListPayoutsGlobalsTypedDict(TypedDict):
    merchant_account_id: NotRequired[str]
    r"""The ID of the merchant account to use for this request."""


class ListPayoutsGlobals(BaseModel):
    merchant_account_id: Annotated[
        Optional[str],
        pydantic.Field(alias="x-gr4vy-merchant-account-id"),
        FieldMetadata(header=HeaderMetadata(style="simple", explode=False)),
    ] = None
    r"""The ID of the merchant account to use for this request."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["merchant_account_id"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class ListPayoutsRequestTypedDict(TypedDict):
    cursor: NotRequired[Nullable[str]]
    r"""A pointer to the page of results to return."""
    limit: NotRequired[int]
    r"""The maximum number of items that are at returned."""
    merchant_account_id: NotRequired[str]
    r"""The ID of the merchant account to use for this request."""


class ListPayoutsRequest(BaseModel):
    cursor: Annotated[
        OptionalNullable[str],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""A pointer to the page of results to return."""

    limit: Annotated[
        Optional[int],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = 20
    r"""The maximum number of items that are at returned."""

    merchant_account_id: Annotated[
        Optional[str],
        pydantic.Field(alias="x-gr4vy-merchant-account-id"),
        FieldMetadata(header=HeaderMetadata(style="simple", explode=False)),
    ] = None
    r"""The ID of the merchant account to use for this request."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["cursor", "limit", "merchant_account_id"])
        nullable_fields = set(["cursor"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            is_nullable_and_explicitly_set = (
                k in nullable_fields
                and (self.__pydantic_fields_set__.intersection({n}))  # pylint: disable=no-member
            )

            if val != UNSET_SENTINEL:
                if (
                    val is not None
                    or k not in optional_fields
                    or is_nullable_and_explicitly_set
                ):
                    m[k] = val

        return m


class ListPayoutsResponseTypedDict(TypedDict):
    result: PayoutSummariesTypedDict


class ListPayoutsResponse(BaseModel):
    next: Callable[[], Optional[ListPayoutsResponse]]

    result: PayoutSummaries



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from gr4vy.types import BaseModel, UnrecognizedStr
from typing import Literal, Union
from typing_extensions import TypedDict


ThreeDSecureScenarioOutcomeAuthenticationTransactionStatus = Union[
    Literal[
        "Y",
        "N",
        "A",
        "R",
        "U",
        "C",
        "timeout",
    ],
    UnrecognizedStr,
]
r"""3DS transaction status."""


class ThreeDSecureScenarioOutcomeAuthenticationTypedDict(TypedDict):
    transaction_status: ThreeDSecureScenarioOutcomeAuthenticationTransactionStatus
    r"""3DS transaction status."""


class ThreeDSecureScenarioOutcomeAuthentication(BaseModel):
    transaction_status: ThreeDSecureScenarioOutcomeAuthenticationTransactionStatus
    r"""3DS transaction status."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .digitalwalletcreate import DigitalWalletCreate, DigitalWalletCreateTypedDict
from gr4vy.types import BaseModel, UNSET_SENTINEL
from gr4vy.utils import FieldMetadata, HeaderMetadata, RequestMetadata
import pydantic
from pydantic import model_serializer
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class ConfigureDigitalWalletGlobalsTypedDict(TypedDict):
    merchant_account_id: NotRequired[str]
    r"""The ID of the merchant account to use for this request."""


class ConfigureDigitalWalletGlobals(BaseModel):
    merchant_account_id: Annotated[
        Optional[str],
        pydantic.Field(alias="x-gr4vy-merchant-account-id"),
        FieldMetadata(header=HeaderMetadata(style="simple", explode=False)),
    ] = None
    r"""The ID of the merchant account to use for this request."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["merchant_account_id"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class ConfigureDigitalWalletRequestTypedDict(TypedDict):
    digital_wallet_create: DigitalWalletCreateTypedDict
    merchant_account_id: NotRequired[str]
    r"""The ID of the merchant account to use for this request."""


class ConfigureDigitalWalletRequest(BaseModel):
    digital_wallet_create: Annotated[
        DigitalWalletCreate,
        FieldMetadata(request=RequestMetadata(media_type="application/json")),
    ]

    merchant_account_id: Annotated[
        Optional[str],
        pydantic.Field(alias="x-gr4vy-merchant-account-id"),
        FieldMetadata(header=HeaderMetadata(style="simple", explode=False)),
    ] = None
    r"""The ID of the merchant account to use for this request."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["merchant_account_id"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .createsessionstatus import CreateSessionStatus
from gr4vy.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from gr4vy.utils import validate_const
import pydantic
from pydantic import model_serializer
from pydantic.functional_validators import AfterValidator
from typing import Any, Dict, Literal, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class CreateSessionTypedDict(TypedDict):
    r"""The session data received from the payment service."""

    status: CreateSessionStatus
    type: Literal["payment-service-session"]
    r"""Always `payment-service-session`."""
    code: NotRequired[Nullable[str]]
    r"""A generic error code that may be returned when the session could not be generated."""
    status_code: NotRequired[Nullable[int]]
    r"""The HTTP status code received from the payment service."""
    response_body: NotRequired[Nullable[Dict[str, Any]]]
    r"""The JSON response body received from the payment service."""


class CreateSession(BaseModel):
    r"""The session data received from the payment service."""

    status: CreateSessionStatus

    TYPE: Annotated[
        Annotated[
            Optional[Literal["payment-service-session"]],
            AfterValidator(validate_const("payment-service-session")),
        ],
        pydantic.Field(alias="type"),
    ] = "payment-service-session"
    r"""Always `payment-service-session`."""

    code: OptionalNullable[str] = UNSET
    r"""A generic error code that may be returned when the session could not be generated."""

    status_code: OptionalNullable[int] = UNSET
    r"""The HTTP status code received from the payment service."""

    response_body: OptionalNullable[Dict[str, Any]] = UNSET
    r"""The JSON response body received from the payment service."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["type", "code", "status_code", "response_body"])
        nullable_fields = set(["code", "status_code", "response_body"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            is_nullable_and_explicitly_set = (
                k in nullable_fields
                and (self.__pydantic_fields_set__.intersection({n}))  # pylint: disable=no-member
            )

            if val != UNSET_SENTINEL:
                if (
                    val is not None
                    or k not in optional_fields
                    or is_nullable_and_explicitly_set
                ):
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .adyencardautorescuescenariosenum import AdyenCardAutoRescueScenariosEnum
from .adyensplitsoptions import AdyenSplitsOptions, AdyenSplitsOptionsTypedDict
from gr4vy.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import Dict
from typing_extensions import Annotated, NotRequired, TypedDict


class AdyenCardOptionsTypedDict(TypedDict):
    auto_rescue: NotRequired[Nullable[bool]]
    r"""Set to `true` to enable Auto Rescue for a transaction. Use the `maxDaysToRescue` to specify a rescue window."""
    max_days_to_rescue: NotRequired[Nullable[int]]
    r"""The rescue window for a transaction, in days, when `autoRescue` is set to `true`. You can specify a value between 1 and 48. For cards, the default is one calendar month. For SEPA, the default is 42 days."""
    additional_data: NotRequired[Nullable[Dict[str, str]]]
    r"""Passes additional data to the Adyen API when creating a transaction."""
    auto_rescue_scenario: NotRequired[Nullable[AdyenCardAutoRescueScenariosEnum]]
    r"""The rescue scenario to simulate for a transaction, when `autoRescue` is set to `true`."""
    window_origin: NotRequired[Nullable[str]]
    r"""The origin of the window where the payment is initiated, used for 3D Secure authentication."""
    splits: NotRequired[Nullable[AdyenSplitsOptionsTypedDict]]
    r"""Passes information of splitting payment amounts to the Adyen API."""


class AdyenCardOptions(BaseModel):
    auto_rescue: Annotated[
        OptionalNullable[bool], pydantic.Field(alias="autoRescue")
    ] = UNSET
    r"""Set to `true` to enable Auto Rescue for a transaction. Use the `maxDaysToRescue` to specify a rescue window."""

    max_days_to_rescue: Annotated[
        OptionalNullable[int], pydantic.Field(alias="maxDaysToRescue")
    ] = UNSET
    r"""The rescue window for a transaction, in days, when `autoRescue` is set to `true`. You can specify a value between 1 and 48. For cards, the default is one calendar month. For SEPA, the default is 42 days."""

    additional_data: Annotated[
        OptionalNullable[Dict[str, str]], pydantic.Field(alias="additionalData")
    ] = UNSET
    r"""Passes additional data to the Adyen API when creating a transaction."""

    auto_rescue_scenario: Annotated[
        OptionalNullable[AdyenCardAutoRescueScenariosEnum],
        pydantic.Field(alias="autoRescueScenario"),
    ] = UNSET
    r"""The rescue scenario to simulate for a transaction, when `autoRescue` is set to `true`."""

    window_origin: OptionalNullable[str] = UNSET
    r"""The origin of the window where the payment is initiated, used for 3D Secure authentication."""

    splits: OptionalNullable[AdyenSplitsOptions] = UNSET
    r"""Passes information of splitting payment amounts to the Adyen API."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "autoRescue",
                "maxDaysToRescue",
                "additionalData",
                "autoRescueScenario",
                "window_origin",
                "splits",
            ]
        )
        nullable_fields = set(
            [
                "autoRescue",
                "maxDaysToRescue",
                "additionalData",
                "autoRescueScenario",
                "window_origin",
                "splits",
            ]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            is_nullable_and_explicitly_set = (
                k in nullable_fields
                and (self.__pydantic_fields_set__.intersection({n}))  # pylint: disable=no-member
            )

            if val != UNSET_SENTINEL:
                if (
                    val is not None
                    or k not in optional_fields
                    or is_nullable_and_explicitly_set
                ):
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from gr4vy.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from gr4vy.utils import validate_const
import pydantic
from pydantic import model_serializer
from pydantic.functional_validators import AfterValidator
from typing import Literal, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class TokenPaymentMethodCreateTypedDict(TypedDict):
    r"""Create a transaction with the ID (token) of a stored payment method (and an optional URL for approval)"""

    id: str
    r"""The ID for the payment method."""
    method: Literal["id"]
    r"""Always `id`."""
    security_code: NotRequired[Nullable[str]]
    r"""The 3 or 4 digit security code often found on the card. This often referred to as the CVV or CVD."""
    redirect_url: NotRequired[Nullable[str]]
    r"""The URL to redirect a user back to after they approve the transaction in the browser."""


class TokenPaymentMethodCreate(BaseModel):
    r"""Create a transaction with the ID (token) of a stored payment method (and an optional URL for approval)"""

    id: str
    r"""The ID for the payment method."""

    METHOD: Annotated[
        Annotated[Optional[Literal["id"]], AfterValidator(validate_const("id"))],
        pydantic.Field(alias="method"),
    ] = "id"
    r"""Always `id`."""

    security_code: OptionalNullable[str] = UNSET
    r"""The 3 or 4 digit security code often found on the card. This often referred to as the CVV or CVD."""

    redirect_url: OptionalNullable[str] = UNSET
    r"""The URL to redirect a user back to after they approve the transaction in the browser."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["method", "security_code", "redirect_url"])
        nullable_fields = set(["security_code", "redirect_url"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            is_nullable_and_explicitly_set = (
                k in nullable_fields
                and (self.__pydantic_fields_set__.intersection({n}))  # pylint: disable=no-member
            )

            if val != UNSET_SENTINEL:
                if (
                    val is not None
                    or k not in optional_fields
                    or is_nullable_and_explicitly_set
                ):
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from gr4vy.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from pydantic import model_serializer
from typing_extensions import NotRequired, TypedDict


class ForterAntiFraudOptionsCartItemBeneficiaryAddressTypedDict(TypedDict):
    country: str
    r"""The country code of the beneficiary's address."""
    address1: NotRequired[Nullable[str]]
    r"""First line of the beneficiary's address."""
    address2: NotRequired[Nullable[str]]
    r"""Second line of the beneficiary's address."""
    zip: NotRequired[Nullable[str]]
    r"""Zip or postal code of the beneficiary's address."""
    region: NotRequired[Nullable[str]]
    r"""State or region of the beneficiary's address."""
    company: NotRequired[Nullable[str]]
    r"""Company name associated with the beneficiary's address."""
    city: NotRequired[Nullable[str]]
    r"""City of the beneficiary's address."""


class ForterAntiFraudOptionsCartItemBeneficiaryAddress(BaseModel):
    country: str
    r"""The country code of the beneficiary's address."""

    address1: OptionalNullable[str] = UNSET
    r"""First line of the beneficiary's address."""

    address2: OptionalNullable[str] = UNSET
    r"""Second line of the beneficiary's address."""

    zip: OptionalNullable[str] = UNSET
    r"""Zip or postal code of the beneficiary's address."""

    region: OptionalNullable[str] = UNSET
    r"""State or region of the beneficiary's address."""

    company: OptionalNullable[str] = UNSET
    r"""Company name associated with the beneficiary's address."""

    city: OptionalNullable[str] = UNSET
    r"""City of the beneficiary's address."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            ["address1", "address2", "zip", "region", "company", "city"]
        )
        nullable_fields = set(
            ["address1", "address2", "zip", "region", "company", "city"]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            is_nullable_and_explicitly_set = (
                k in nullable_fields
                and (self.__pydantic_fields_set__.intersection({n}))  # pylint: disable=no-member
            )

            if val != UNSET_SENTINEL:
                if (
                    val is not None
                    or k not in optional_fields
                    or is_nullable_and_explicitly_set
                ):
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from gr4vy.types import BaseModel, UNSET_SENTINEL
from gr4vy.utils import validate_const
import pydantic
from pydantic import model_serializer
from pydantic.functional_validators import AfterValidator
from typing import Literal, Optional
from typing_extensions import Annotated, TypedDict


class PaymentMethodStoredCardTypedDict(TypedDict):
    id: str
    r"""The ID of the stored card to use."""
    method: Literal["id"]
    r"""Set to `id` to use a stored card."""


class PaymentMethodStoredCard(BaseModel):
    id: str
    r"""The ID of the stored card to use."""

    METHOD: Annotated[
        Annotated[Optional[Literal["id"]], AfterValidator(validate_const("id"))],
        pydantic.Field(alias="method"),
    ] = "id"
    r"""Set to `id` to use a stored card."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["method"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from gr4vy.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from pydantic import model_serializer
from typing_extensions import NotRequired, TypedDict


class ThreeDSecureScenarioConditionsTypedDict(TypedDict):
    first_name: NotRequired[Nullable[str]]
    r"""First name of the buyer to match."""
    last_name: NotRequired[Nullable[str]]
    r"""Last name of the buyer to match."""
    email_address: NotRequired[Nullable[str]]
    r"""Email address of the buyer to match."""
    amount: NotRequired[Nullable[int]]
    r"""Amount of the transaction to match."""
    external_identifier: NotRequired[Nullable[str]]
    r"""External identifier to match."""
    card_number: NotRequired[Nullable[str]]
    r"""Card number to match."""


class ThreeDSecureScenarioConditions(BaseModel):
    first_name: OptionalNullable[str] = UNSET
    r"""First name of the buyer to match."""

    last_name: OptionalNullable[str] = UNSET
    r"""Last name of the buyer to match."""

    email_address: OptionalNullable[str] = UNSET
    r"""Email address of the buyer to match."""

    amount: OptionalNullable[int] = UNSET
    r"""Amount of the transaction to match."""

    external_identifier: OptionalNullable[str] = UNSET
    r"""External identifier to match."""

    card_number: OptionalNullable[str] = UNSET
    r"""Card number to match."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "first_name",
                "last_name",
                "email_address",
                "amount",
                "external_identifier",
                "card_number",
            ]
        )
        nullable_fields = set(
            [
                "first_name",
                "last_name",
                "email_address",
                "amount",
                "external_identifier",
                "card_number",
            ]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            is_nullable_and_explicitly_set = (
                k in nullable_fields
                and (self.__pydantic_fields_set__.intersection({n}))  # pylint: disable=no-member
            )

            if val != UNSET_SENTINEL:
                if (
                    val is not None
                    or k not in optional_fields
                    or is_nullable_and_explicitly_set
                ):
                    m[k] = val

        return m
